@@ -198,3 +198,15 @@ ACE_SET_AUDIT_FAILURE = 6
 # VLAN operation modes
 VLAN_MODE_ACCESS = 1
 VLAN_MODE_TRUNK = 2
+
+# Action that Hyper-V takes on the VM
+# when the host is shut down.
+HOST_SHUTDOWN_ACTION_TURN_OFF = 2
+HOST_SHUTDOWN_ACTION_SAVE = 3
+HOST_SHUTDOWN_ACTION_SHUTDOWN = 4
+
+# VM snapshot types
+VM_SNAPSHOT_TYPE_DISABLED = 2
+VM_SNAPSHOT_TYPE_PROD_FALLBACK = 3
+VM_SNAPSHOT_TYPE_PROD_ENFORCED = 4
+VM_SNAPSHOT_TYPE_STANDARD = 5
