@@ -24,13 +24,21 @@ from os_win._i18n import _
 # Define WMI specific exceptions, so WMI won't have to be imported in any
 # module that expects those exceptions.
 if sys.platform == 'win32':
+    from six.moves.builtins import WindowsError
     import wmi
 
     x_wmi = wmi.x_wmi
     x_wmi_timed_out = wmi.x_wmi_timed_out
 else:
+    class WindowsError(Exception):
+        def __init__(self, winerror=None):
+            self.winerror = winerror
+
     class x_wmi(Exception):
-        pass
+        def __init__(self, info='', com_error=None):
+            super(x_wmi, self).__init__()
+            self.info = info
+            self.com_error = com_error
 
     class x_wmi_timed_out(x_wmi):
         pass
@@ -41,6 +49,7 @@ class OSWinException(Exception):
 
     def __init__(self, message=None, **kwargs):
         self.kwargs = kwargs
+        self.error_code = kwargs.get('error_code')
 
         if not message:
             message = self.msg_fmt % kwargs
@@ -100,10 +109,6 @@ class Win32Exception(OSWinException):
                 "Error code: %(error_code)s. "
                 "Error message: %(error_message)s")
 
-    def __init__(self, message=None, **kwargs):
-        self.error_code = kwargs.get('error_code')
-        super(Win32Exception, self).__init__(message=message, **kwargs)
-
 
 class VHDException(OSWinException):
     pass
@@ -196,8 +201,36 @@ class ClusterWin32Exception(ClusterException, Win32Exception):
     pass
 
 
+# TODO(lpetrut): Remove this exception in Q. It was never used outside
+# os-win.
 class InvalidClusterGroupState(ClusterException):
     msg_fmt = _("The cluster group %(group_name)s is in an invalid state. "
                 "Expected state %(expected_state)s. Expected owner node: "
                 "%(expected_node)s. Current group state: %(group_state)s. "
                 "Current owner node: %(owner_node)s.")
+
+
+class ClusterGroupMigrationFailed(ClusterException):
+    msg_fmt = _("Failed to migrate cluster group %(group_name)s. "
+                "Expected state %(expected_state)s. "
+                "Expected owner node: %(expected_node)s. "
+                "Current group state: %(group_state)s. "
+                "Current owner node: %(owner_node)s.")
+
+
+class ClusterGroupMigrationTimeOut(ClusterGroupMigrationFailed):
+    msg_fmt = _("Cluster group '%(group_name)s' migration "
+                "timed out after %(time_elapsed)0.3fs. ")
+
+
+class ClusterPropertyRetrieveFailed(ClusterException):
+    msg_fmt = _("Failed to retrieve a cluster property.")
+
+
+class ClusterPropertyListEntryNotFound(ClusterPropertyRetrieveFailed):
+    msg_fmt = _("The specified cluster property list does not contain "
+                "an entry named '%(property_name)s'")
+
+
+class ClusterPropertyListParsingError(ClusterPropertyRetrieveFailed):
+    msg_fmt = _("Parsing a cluster property list failed.")
