@@ -12,7 +12,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ddt
 import mock
+from oslo_utils import units
 
 from os_win import constants
 from os_win import exceptions
@@ -21,6 +23,7 @@ from os_win.utils import _wqlutils
 from os_win.utils.network import networkutils
 
 
+@ddt.ddt
 class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
     """Unit tests for the Hyper-V NetworkUtils class."""
 
@@ -61,7 +64,9 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.netutils._switches = {}
         self.netutils._switch_ports = {}
         self.netutils._vlan_sds = {}
+        self.netutils._profile_sds = {}
         self.netutils._vsid_sds = {}
+        self.netutils._bandwidth_sds = {}
         conn = self.netutils._conn
 
         mock_vswitch = mock.MagicMock(ElementName=mock.sentinel.vswitch_name)
@@ -73,10 +78,14 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         mock_sd = mock.MagicMock(InstanceID=self._FAKE_INSTANCE_ID)
         mock_bad_sd = mock.MagicMock(InstanceID=self._FAKE_BAD_INSTANCE_ID)
+        conn.Msvm_EthernetSwitchPortProfileSettingData.return_value = [
+            mock_bad_sd, mock_sd]
         conn.Msvm_EthernetSwitchPortVlanSettingData.return_value = [
             mock_bad_sd, mock_sd]
         conn.Msvm_EthernetSwitchPortSecuritySettingData.return_value = [
             mock_bad_sd, mock_sd]
+        conn.Msvm_EthernetSwitchPortBandwidthSettingData.return_value = [
+            mock_bad_sd, mock_sd]
 
         self.netutils.init_caches()
 
@@ -84,10 +93,14 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
                          self.netutils._switches)
         self.assertEqual({mock.sentinel.port_name: mock_port},
                          self.netutils._switch_ports)
+        self.assertEqual([mock_sd], list(self.netutils._profile_sds.values()))
         self.assertEqual([mock_sd], list(self.netutils._vlan_sds.values()))
         self.assertEqual([mock_sd], list(self.netutils._vsid_sds.values()))
+        self.assertEqual([mock_sd],
+                         list(self.netutils._bandwidth_sds.values()))
 
     def test_update_cache(self):
+        self.netutils._switch_ports[mock.sentinel.other] = mock.sentinel.port
         conn = self.netutils._conn
         mock_port = mock.MagicMock(ElementName=mock.sentinel.port_name)
         conn.Msvm_EthernetPortAllocationSettingData.return_value = [
@@ -98,6 +111,11 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertEqual({mock.sentinel.port_name: mock_port},
                          self.netutils._switch_ports)
 
+        # assert that other networkutils have the same cache.
+        netutils = networkutils.NetworkUtils()
+        self.assertEqual({mock.sentinel.port_name: mock_port},
+                         netutils._switch_ports)
+
     def test_clear_port_sg_acls_cache(self):
         self.netutils._sg_acl_sds[mock.sentinel.port_id] = [mock.sentinel.acl]
         self.netutils.clear_port_sg_acls_cache(mock.sentinel.port_id)
@@ -283,6 +301,71 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
                           self.netutils._get_vswitch,
                           self._FAKE_VSWITCH_NAME)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_prepare_profile_sd')
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_profile_setting_data_from_port_alloc')
+    def _test_set_vswitch_port_profile_id(
+            self, mock_get_profile_setting_data_from_port_alloc,
+            mock_prepare_profile_sd, found, side_effect=None):
+        mock_port_profile = mock.MagicMock()
+        mock_new_port_profile = mock.MagicMock()
+        mock_port_alloc = self._mock_get_switch_port_alloc()
+
+        mock_add_feature = self.netutils._jobutils.add_virt_feature
+        mock_remove_feature = self.netutils._jobutils.remove_virt_feature
+
+        mock_get_profile_setting_data_from_port_alloc.return_value = (
+            mock_port_profile if found else None
+        )
+        mock_prepare_profile_sd.return_value = mock_new_port_profile
+        mock_add_feature.side_effect = side_effect
+
+        fake_params = {
+            "switch_port_name": self._FAKE_PORT_NAME,
+            "profile_id": mock.sentinel.profile_id,
+            "profile_data": mock.sentinel.profile_data,
+            "profile_name": mock.sentinel.profile_name,
+            "net_cfg_instance_id": None,
+            "cdn_label_id": None,
+            "cdn_label_string": None,
+            "vendor_id": None,
+            "vendor_name": mock.sentinel.vendor_name,
+        }
+
+        if side_effect:
+            self.assertRaises(
+                exceptions.HyperVException,
+                self.netutils.set_vswitch_port_profile_id,
+                **fake_params)
+        else:
+            self.netutils.set_vswitch_port_profile_id(**fake_params)
+
+        fake_params.pop("switch_port_name")
+        mock_prepare_profile_sd.assert_called_once_with(**fake_params)
+
+        if found:
+            mock_remove_feature.assert_called_once_with(mock_port_profile)
+            self.assertNotIn(self._FAKE_INSTANCE_ID,
+                             self.netutils._profile_sds)
+
+        mock_get_profile_setting_data_from_port_alloc.assert_called_with(
+            mock_port_alloc)
+
+        self.assertNotIn(mock_port_alloc, self.netutils._profile_sds)
+        mock_add_feature.assert_called_once_with(mock_new_port_profile,
+                                                 mock_port_alloc)
+
+    def test_set_vswitch_port_profile_id(self):
+        self._test_set_vswitch_port_profile_id(found=True)
+
+    def test_set_vswitch_port_profile_id_not_found(self):
+        self._test_set_vswitch_port_profile_id(found=False)
+
+    def test_set_vswitch_port_profile_id_failed(self):
+        self._test_set_vswitch_port_profile_id(found=False,
+                                               side_effect=Exception)
+
     def test_set_vswitch_port_vlan_id_invalid_mode(self):
         self.assertRaises(
             AttributeError, self.netutils.set_vswitch_port_vlan_id,
@@ -491,6 +574,17 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertFalse(self.netutils._jobutils.add_virt_feature.called)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_setting_data_from_port_alloc')
+    def test_get_profile_setting_data_from_port_alloc(self, mock_get_sd):
+        result = self.netutils._get_profile_setting_data_from_port_alloc(
+            mock.sentinel.port)
+
+        self.assertEqual(mock_get_sd.return_value, result)
+        mock_get_sd.assert_called_once_with(
+            mock.sentinel.port, self.netutils._profile_sds,
+            self.netutils._PORT_PROFILE_SET_DATA)
+
     @mock.patch.object(networkutils.NetworkUtils,
                        '_get_setting_data_from_port_alloc')
     def test_get_vlan_setting_data_from_port_alloc(self, mock_get_sd):
@@ -514,6 +608,18 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
             mock_port, self.netutils._vsid_sds,
             self.netutils._PORT_SECURITY_SET_DATA)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_setting_data_from_port_alloc')
+    def test_get_bandwidth_setting_data_from_port_alloc(self, mock_get_sd):
+        mock_port = mock.MagicMock()
+        result = self.netutils._get_bandwidth_setting_data_from_port_alloc(
+            mock_port)
+
+        self.assertEqual(mock_get_sd.return_value, result)
+        mock_get_sd.assert_called_once_with(
+            mock_port, self.netutils._bandwidth_sds,
+            self.netutils._PORT_BANDWIDTH_SET_DATA)
+
     def test_get_setting_data_from_port_alloc_cached(self):
         mock_port = mock.MagicMock(InstanceID=mock.sentinel.InstanceID)
         cache = {mock_port.InstanceID: mock.sentinel.sd_object}
@@ -808,6 +914,135 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         actual = self.netutils._get_new_weights([mock.ANY, mock.ANY], mock.ANY)
         self.assertEqual([0, 0], actual)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_bandwidth_setting_data_from_port_alloc')
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_default_setting_data')
+    def test_set_port_qos_rule_hyperv_exc(self, mock_get_default_sd,
+                                          mock_get_bandwidth_sd):
+        mock_port_alloc = self._mock_get_switch_port_alloc()
+
+        self.netutils._bandwidth_sds = {
+            mock_port_alloc.InstanceID: mock.sentinel.InstanceID}
+        mock_remove_feature = self.netutils._jobutils.remove_virt_feature
+        mock_add_feature = self.netutils._jobutils.add_virt_feature
+        mock_add_feature.side_effect = exceptions.HyperVException
+
+        qos_rule = dict(min_kbps=20000, max_kbps=30000,
+                        max_burst_kbps=40000, max_burst_size_kb=50000)
+
+        self.assertRaises(exceptions.HyperVException,
+                          self.netutils.set_port_qos_rule,
+                          mock.sentinel.port_id, qos_rule)
+
+        mock_get_bandwidth_sd.assert_called_once_with(mock_port_alloc)
+        mock_get_default_sd.assert_called_once_with(
+            self.netutils._PORT_BANDWIDTH_SET_DATA)
+        mock_remove_feature.assert_called_once_with(
+            mock_get_bandwidth_sd.return_value)
+        mock_add_feature.assert_called_once_with(
+            mock_get_default_sd.return_value, mock_port_alloc)
+
+        bw = mock_get_default_sd.return_value
+        self.assertEqual(qos_rule['min_kbps'] * units.Ki,
+                         bw.Reservation)
+        self.assertEqual(qos_rule['max_kbps'] * units.Ki,
+                         bw.Limit)
+        self.assertEqual(qos_rule['max_burst_kbps'] * units.Ki,
+                         bw.BurstLimit)
+        self.assertEqual(qos_rule['max_burst_size_kb'] * units.Ki,
+                         bw.BurstSize)
+        self.assertNotIn(mock_port_alloc.InstanceID,
+                         self.netutils._bandwidth_sds)
+
+    @ddt.data({'min_kbps': 100},
+              {'min_kbps': 10 * units.Ki, 'max_kbps': 100},
+              {'max_kbps': 10 * units.Ki, 'max_burst_kbps': 100})
+    def test_set_port_qos_rule_invalid_params_exception(self, qos_rule):
+        self.assertRaises(exceptions.InvalidParameterValue,
+                          self.netutils.set_port_qos_rule,
+                          mock.sentinel.port_id,
+                          qos_rule)
+
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_bandwidth_setting_data_from_port_alloc')
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_default_setting_data')
+    def test_set_port_qos_rule_invalid_qos_rule_exc(self, mock_get_default_sd,
+                                                    mock_get_bandwidth_sd):
+        self._mock_get_switch_port_alloc()
+
+        mock_add_feature = self.netutils._jobutils.add_virt_feature
+        mock_add_feature.side_effect = exceptions.InvalidParameterValue(
+            '0x80070057')
+
+        qos_rule = dict(min_kbps=20000, max_kbps=30000,
+                        max_burst_kbps=40000, max_burst_size_kb=50000)
+
+        self.assertRaises(exceptions.InvalidParameterValue,
+                          self.netutils.set_port_qos_rule,
+                          mock.sentinel.port_id, qos_rule)
+
+    def test_set_empty_port_qos_rule(self):
+        self._mock_get_switch_port_alloc()
+
+        self.netutils.set_port_qos_rule(mock.sentinel.port_id, {})
+        self.assertFalse(self.netutils._get_switch_port_allocation.called)
+
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_bandwidth_setting_data_from_port_alloc')
+    def test_remove_port_qos_rule(self, mock_get_bandwidth_sd):
+        mock_port_alloc = self._mock_get_switch_port_alloc()
+        mock_bandwidth_settings = mock_get_bandwidth_sd.return_value
+
+        self.netutils.remove_port_qos_rule(mock.sentinel.port_id)
+
+        mock_get_bandwidth_sd.assert_called_once_with(mock_port_alloc)
+        mock_remove_feature = self.netutils._jobutils.remove_virt_feature
+        mock_remove_feature.assert_called_once_with(
+            mock_bandwidth_settings)
+
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_create_default_setting_data')
+    def test_prepare_profile_sd(self, mock_create_default_sd):
+        mock_profile_sd = mock_create_default_sd.return_value
+
+        actual_profile_sd = self.netutils._prepare_profile_sd(
+            profile_id=mock.sentinel.profile_id,
+            profile_data=mock.sentinel.profile_data,
+            profile_name=mock.sentinel.profile_name,
+            net_cfg_instance_id=mock.sentinel.net_cfg_instance_id,
+            cdn_label_id=mock.sentinel.cdn_label_id,
+            cdn_label_string=mock.sentinel.cdn_label_string,
+            vendor_id=mock.sentinel.vendor_id,
+            vendor_name=mock.sentinel.vendor_name)
+
+        self.assertEqual(mock_profile_sd, actual_profile_sd)
+        self.assertEqual(mock.sentinel.profile_id,
+                         mock_profile_sd.ProfileId)
+        self.assertEqual(mock.sentinel.profile_data,
+                         mock_profile_sd.ProfileData)
+        self.assertEqual(mock.sentinel.profile_name,
+                         mock_profile_sd.ProfileName)
+        self.assertEqual(mock.sentinel.net_cfg_instance_id,
+                         mock_profile_sd.NetCfgInstanceId)
+        self.assertEqual(mock.sentinel.cdn_label_id,
+                         mock_profile_sd.CdnLabelId)
+        self.assertEqual(mock.sentinel.cdn_label_string,
+                         mock_profile_sd.CdnLabelString)
+        self.assertEqual(mock.sentinel.vendor_id,
+                         mock_profile_sd.VendorId)
+        self.assertEqual(mock.sentinel.vendor_name,
+                         mock_profile_sd.VendorName)
+        mock_create_default_sd.assert_called_once_with(
+            self.netutils._PORT_PROFILE_SET_DATA)
+
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_create_default_setting_data')
+    def test_prepare_profile_sd_failed(self, mock_create_default_sd):
+        self.assertRaises(TypeError, self.netutils._prepare_profile_sd,
+                          invalid_argument=mock.sentinel.invalid_argument)
+
 
 class TestNetworkUtilsR2(test_base.OsWinBaseTestCase):
 
