@@ -42,6 +42,9 @@ from os_win.utils import pathutils
 
 LOG = logging.getLogger(__name__)
 
+# TODO(claudiub): remove the is_planned_vm argument from methods once it is not
+# used  anymore.
+
 
 class VMUtils(baseutils.BaseUtilsVirt):
 
@@ -68,13 +71,16 @@ class VMUtils(baseutils.BaseUtilsVirt):
     _CIM_RES_ALLOC_SETTING_DATA_CLASS = 'Cim_ResourceAllocationSettingData'
     _COMPUTER_SYSTEM_CLASS = "Msvm_ComputerSystem"
     _LOGICAL_IDENTITY_CLASS = 'Msvm_LogicalIdentity'
+    _VIRTUAL_SYSTEM_SNAP_ASSOC_CLASS = 'Msvm_SnapshotOfVirtualSystem'
 
     _S3_DISP_CTRL_RES_SUB_TYPE = 'Microsoft:Hyper-V:S3 Display Controller'
     _SYNTH_DISP_CTRL_RES_SUB_TYPE = ('Microsoft:Hyper-V:Synthetic Display '
                                      'Controller')
-    _SYNTH_3D_DISP_CTRL_RES_SUB_TYPE = ('Microsoft:Hyper-V:Synthetic 3D '
+    _REMOTEFX_DISP_CTRL_RES_SUB_TYPE = ('Microsoft:Hyper-V:Synthetic 3D '
                                         'Display Controller')
-    _SYNTH_3D_DISP_ALLOCATION_SETTING_DATA_CLASS = (
+    _SYNTH_DISP_ALLOCATION_SETTING_DATA_CLASS = (
+        'Msvm_SyntheticDisplayControllerSettingData')
+    _REMOTEFX_DISP_ALLOCATION_SETTING_DATA_CLASS = (
         'Msvm_Synthetic3DDisplayControllerSettingData')
 
     _VIRTUAL_SYSTEM_SUBTYPE = 'VirtualSystemSubType'
@@ -109,14 +115,19 @@ class VMUtils(baseutils.BaseUtilsVirt):
     }
 
     _DISP_CTRL_ADDRESS_DX_11 = "02C1,00000000,01"
+    _DISP_CTRL_ADDRESS = "5353,00000000,00"
 
     _vm_power_states_map = {constants.HYPERV_VM_STATE_ENABLED: 2,
                             constants.HYPERV_VM_STATE_DISABLED: 3,
-                            constants.HYPERV_VM_STATE_SHUTTING_DOWN: 4,
                             constants.HYPERV_VM_STATE_REBOOT: 11,
                             constants.HYPERV_VM_STATE_PAUSED: 9,
                             constants.HYPERV_VM_STATE_SUSPENDED: 6}
 
+    _disk_ctrl_type_mapping = {
+        _SCSI_CTRL_RES_SUB_TYPE: constants.CTRL_TYPE_SCSI,
+        _IDE_CTRL_RES_SUB_TYPE: constants.CTRL_TYPE_IDE
+    }
+
     _DEFAULT_EVENT_CHECK_TIMEFRAME = 60  # seconds
     _DEFAULT_EVENT_TIMEOUT_MS = 2000
 
@@ -191,21 +202,19 @@ class VMUtils(baseutils.BaseUtilsVirt):
         settings = self.get_vm_summary_info(vm_name)
         return settings['EnabledState']
 
-    def _lookup_vm_check(self, vm_name, as_vssd=True, for_update=False,
-                         virtual_system_type=_VIRTUAL_SYSTEM_TYPE_REALIZED):
-        vm = self._lookup_vm(vm_name, as_vssd, for_update,
-                             virtual_system_type)
+    def _lookup_vm_check(self, vm_name, as_vssd=True, for_update=False):
+        vm = self._lookup_vm(vm_name, as_vssd, for_update)
         if not vm:
             raise exceptions.HyperVVMNotFoundException(vm_name=vm_name)
         return vm
 
-    def _lookup_vm(self, vm_name, as_vssd=True, for_update=False,
-                   virtual_system_type=_VIRTUAL_SYSTEM_TYPE_REALIZED):
+    def _lookup_vm(self, vm_name, as_vssd=True, for_update=False):
         if as_vssd:
             conn = self._compat_conn if for_update else self._conn
-            vms = conn.Msvm_VirtualSystemSettingData(
-                ElementName=vm_name,
-                VirtualSystemType=virtual_system_type)
+            vms = conn.Msvm_VirtualSystemSettingData(ElementName=vm_name)
+            vms = [v for v in vms if
+                   v.VirtualSystemType in [self._VIRTUAL_SYSTEM_TYPE_PLANNED,
+                                           self._VIRTUAL_SYSTEM_TYPE_REALIZED]]
         else:
             vms = self._conn.Msvm_ComputerSystem(ElementName=vm_name)
         n = len(vms)
@@ -282,28 +291,32 @@ class VMUtils(baseutils.BaseUtilsVirt):
     def update_vm(self, vm_name, memory_mb, memory_per_numa_node, vcpus_num,
                   vcpus_per_numa_node, limit_cpu_features, dynamic_mem_ratio,
                   configuration_root_dir=None, snapshot_dir=None,
-                  host_shutdown_action=None,
+                  host_shutdown_action=None, vnuma_enabled=None,
+                  snapshot_type=constants.VM_SNAPSHOT_TYPE_PROD_FALLBACK,
                   is_planned_vm=False):
-        virtual_system_type = self._get_virtual_system_type(is_planned_vm)
-
-        vmsetting = self._lookup_vm_check(
-            vm_name, virtual_system_type=virtual_system_type)
+        vmsetting = self._lookup_vm_check(vm_name, for_update=True)
 
         if host_shutdown_action:
             vmsetting.AutomaticShutdownAction = host_shutdown_action
         if configuration_root_dir:
+            # Created VMs must have their *DataRoot paths in the same location
+            # as the VM's path.
             vmsetting.ConfigurationDataRoot = configuration_root_dir
-        if snapshot_dir:
-            vmsetting.SnapshotDataRoot = snapshot_dir
+            vmsetting.LogDataRoot = configuration_root_dir
+            vmsetting.SnapshotDataRoot = configuration_root_dir
+            vmsetting.SuspendDataRoot = configuration_root_dir
+            vmsetting.SwapFileDataRoot = configuration_root_dir
+        if vnuma_enabled is not None:
+            vmsetting.VirtualNumaEnabled = vnuma_enabled
+
         self._set_vm_memory(vmsetting, memory_mb, memory_per_numa_node,
                             dynamic_mem_ratio)
         self._set_vm_vcpus(vmsetting, vcpus_num, vcpus_per_numa_node,
                            limit_cpu_features)
 
-        update_needed = (configuration_root_dir or snapshot_dir or
-                         host_shutdown_action)
-        if update_needed:
-            self._modify_virtual_system(vmsetting)
+        self._set_vm_snapshot_type(vmsetting, snapshot_type)
+
+        self._modify_virtual_system(vmsetting)
 
     def check_admin_permissions(self):
         if not self._compat_conn.Msvm_VirtualSystemManagementService():
@@ -484,8 +497,45 @@ class VMUtils(baseutils.BaseUtilsVirt):
         res.Parent = drive_path
         res.HostResource = [path]
 
-        # Add the new vhd object as a virtual hard disk to the vm.
-        self._jobutils.add_virt_resource(res, vm)
+        try:
+            # Add the new vhd object as a virtual hard disk to the vm.
+            self._jobutils.add_virt_resource(res, vm)
+        except Exception:
+            LOG.exception(_LE("Failed to attach disk image %(disk_path)s "
+                              "to vm %(vm_name)s. Reverting attachment."),
+                          dict(disk_path=path, vm_name=vm_name))
+
+            drive = self._get_wmi_obj(drive_path)
+            self._jobutils.remove_virt_resource(drive)
+            raise
+
+    def get_disk_attachment_info(self, attached_disk_path, is_physical=True):
+        res = self._get_mounted_disk_resource_from_path(attached_disk_path,
+                                                        is_physical)
+        if not res:
+            err_msg = _("Disk '%s' is not attached to a vm.")
+            raise exceptions.DiskNotFound(err_msg % attached_disk_path)
+
+        if is_physical:
+            drive = res
+        else:
+            drive = self._get_wmi_obj(res.Parent)
+
+        ctrl_slot = int(drive.AddressOnParent)
+        ctrl_path = drive.Parent
+        ctrl_type = self._get_disk_controller_type(ctrl_path)
+
+        attachment_info = dict(controller_slot=ctrl_slot,
+                               controller_path=ctrl_path,
+                               controller_type=ctrl_type)
+        return attachment_info
+
+    def _get_disk_controller_type(self, controller_path):
+        ctrl = self._get_wmi_obj(controller_path)
+        res_sub_type = ctrl.ResourceSubType
+
+        ctrl_type = self._disk_ctrl_type_mapping[res_sub_type]
+        return ctrl_type
 
     def create_scsi_controller(self, vm_name):
         """Create an iscsi controller ready to mount volumes."""
@@ -522,7 +572,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
     def get_vm_physical_disk_mapping(self, vm_name, is_planned_vm=False):
         mapping = {}
         physical_disks = (
-            self.get_vm_disks(vm_name, is_planned_vm=is_planned_vm)[1])
+            self.get_vm_disks(vm_name)[1])
         for diskdrive in physical_disks:
             mapping[diskdrive.ElementName] = dict(
                 resource_path=diskdrive.path_(),
@@ -632,10 +682,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
         return vmsettings.ConfigurationDataRoot
 
     def get_vm_storage_paths(self, vm_name, is_planned_vm=False):
-        virtual_system_type = self._get_virtual_system_type(is_planned_vm)
-        vmsettings = self._lookup_vm_check(
-            vm_name,
-            virtual_system_type=virtual_system_type)
+        vmsettings = self._lookup_vm_check(vm_name)
         (disk_resources, volume_resources) = self._get_vm_disks(vmsettings)
 
         volume_drives = []
@@ -651,9 +698,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
         return (disk_files, volume_drives)
 
     def get_vm_disks(self, vm_name, is_planned_vm=False):
-        virtual_system_type = self._get_virtual_system_type(is_planned_vm)
-        vmsettings = self._lookup_vm_check(
-            vm_name, virtual_system_type=virtual_system_type)
+        vmsettings = self._lookup_vm_check(vm_name)
         return self._get_vm_disks(vmsettings)
 
     def _get_vm_disks(self, vmsettings):
@@ -683,21 +728,34 @@ class VMUtils(baseutils.BaseUtilsVirt):
         (job_path, ret_val) = self._vs_man_svc.DestroySystem(vm.path_())
         self._jobutils.check_ret_val(ret_val, job_path)
 
-    def take_vm_snapshot(self, vm_name):
+    def take_vm_snapshot(self, vm_name, snapshot_name=None):
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
         vs_snap_svc = self._compat_conn.Msvm_VirtualSystemSnapshotService()[0]
 
         (job_path, snp_setting_data, ret_val) = vs_snap_svc.CreateSnapshot(
             AffectedSystem=vm.path_(),
             SnapshotType=self._SNAPSHOT_FULL)
-        self._jobutils.check_ret_val(ret_val, job_path)
 
-        vm_path = vm.path_().lower()
-        current_snapshots = self._conn.Msvm_MostCurrentSnapshotInBranch()
-        snp_setting_data = [s for s in current_snapshots if
-                            s.Antecedent.path_().lower() == vm_path][0]
+        job = self._jobutils.check_ret_val(ret_val, job_path)
+        snp_setting_data = job.associators(
+            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS,
+            wmi_association_class=self._AFFECTED_JOB_ELEMENT_CLASS)[0]
 
-        return snp_setting_data.Dependent.path_()
+        if snapshot_name is not None:
+            snp_setting_data.ElementName = snapshot_name
+            self._modify_virtual_system(snp_setting_data)
+
+        return snp_setting_data.path_()
+
+    def get_vm_snapshots(self, vm_name, snapshot_name=None):
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
+        snapshots = vm.associators(
+            wmi_association_class=self._VIRTUAL_SYSTEM_SNAP_ASSOC_CLASS,
+            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
+
+        return [
+            s.path_() for s in snapshots
+            if snapshot_name is None or s.ElementName == snapshot_name]
 
     def remove_vm_snapshot(self, snapshot_path):
         vs_snap_svc = self._compat_conn.Msvm_VirtualSystemSnapshotService()[0]
@@ -1070,44 +1128,86 @@ class VMUtils(baseutils.BaseUtilsVirt):
                    'max_monitors':
                    self._remotefx_max_monitors_map[max_resolution]})
 
-    def _add_3d_display_controller(self, vm, monitor_count,
-                                   max_resolution, vram_bytes=None):
-        synth_3d_disp_ctrl_res = self._get_new_resource_setting_data(
-            self._SYNTH_3D_DISP_CTRL_RES_SUB_TYPE,
-            self._SYNTH_3D_DISP_ALLOCATION_SETTING_DATA_CLASS)
+    def _set_remotefx_display_controller(self, vm, remotefx_disp_ctrl_res,
+                                         monitor_count, max_resolution,
+                                         vram_bytes=None):
+        new_wmi_obj = False
+        if not remotefx_disp_ctrl_res:
+            new_wmi_obj = True
+            remotefx_disp_ctrl_res = self._get_new_resource_setting_data(
+                self._REMOTEFX_DISP_CTRL_RES_SUB_TYPE,
+                self._REMOTEFX_DISP_ALLOCATION_SETTING_DATA_CLASS)
 
-        synth_3d_disp_ctrl_res.MaximumMonitors = monitor_count
-        synth_3d_disp_ctrl_res.MaximumScreenResolution = max_resolution
+        remotefx_disp_ctrl_res.MaximumMonitors = monitor_count
+        remotefx_disp_ctrl_res.MaximumScreenResolution = max_resolution
+        self._set_remotefx_vram(remotefx_disp_ctrl_res, vram_bytes)
 
-        self._jobutils.add_virt_resource(synth_3d_disp_ctrl_res, vm)
+        if new_wmi_obj:
+            self._jobutils.add_virt_resource(remotefx_disp_ctrl_res, vm)
+        else:
+            self._jobutils.modify_virt_resource(remotefx_disp_ctrl_res)
+
+    def _set_remotefx_vram(self, remotefx_disp_ctrl_res, vram_bytes):
+        pass
 
     def enable_remotefx_video_adapter(self, vm_name, monitor_count,
                                       max_resolution, vram_bytes=None):
-        vm = self._lookup_vm_check(vm_name, as_vssd=False)
-
         self._validate_remotefx_params(monitor_count, max_resolution,
                                        vram_bytes=vram_bytes)
 
+        vm = self._lookup_vm_check(vm_name)
         rasds = _wqlutils.get_element_associated_class(
             self._compat_conn, self._CIM_RES_ALLOC_SETTING_DATA_CLASS,
-            element_uuid=vm.Name)
-        if [r for r in rasds if r.ResourceSubType ==
-                self._SYNTH_3D_DISP_CTRL_RES_SUB_TYPE]:
-            raise exceptions.HyperVRemoteFXException(
-                _("RemoteFX is already configured for this VM"))
+            element_instance_id=vm.InstanceID)
 
         synth_disp_ctrl_res_list = [r for r in rasds if r.ResourceSubType ==
                                     self._SYNTH_DISP_CTRL_RES_SUB_TYPE]
         if synth_disp_ctrl_res_list:
+            # we need to remove the generic display controller first.
             self._jobutils.remove_virt_resource(synth_disp_ctrl_res_list[0])
 
+        remotefx_disp_ctrl_res = [r for r in rasds if r.ResourceSubType ==
+                                  self._REMOTEFX_DISP_CTRL_RES_SUB_TYPE]
+        remotefx_disp_ctrl_res = (remotefx_disp_ctrl_res[0]
+                                  if remotefx_disp_ctrl_res else None)
+
         max_res_value = self._remote_fx_res_map.get(max_resolution)
-        self._add_3d_display_controller(vm, monitor_count, max_res_value,
-                                        vram_bytes)
-        if self._vm_has_s3_controller(vm.ElementName):
+        self._set_remotefx_display_controller(
+            vm, remotefx_disp_ctrl_res, monitor_count, max_res_value,
+            vram_bytes)
+
+        if self._vm_has_s3_controller(vm_name):
             s3_disp_ctrl_res = [r for r in rasds if r.ResourceSubType ==
                                 self._S3_DISP_CTRL_RES_SUB_TYPE][0]
-            s3_disp_ctrl_res.Address = self._DISP_CTRL_ADDRESS_DX_11
+            if s3_disp_ctrl_res.Address != self._DISP_CTRL_ADDRESS_DX_11:
+                s3_disp_ctrl_res.Address = self._DISP_CTRL_ADDRESS_DX_11
+                self._jobutils.modify_virt_resource(s3_disp_ctrl_res)
+
+    def disable_remotefx_video_adapter(self, vm_name):
+        vm = self._lookup_vm_check(vm_name)
+        rasds = _wqlutils.get_element_associated_class(
+            self._compat_conn, self._CIM_RES_ALLOC_SETTING_DATA_CLASS,
+            element_instance_id=vm.InstanceID)
+
+        remotefx_disp_ctrl_res = [r for r in rasds if r.ResourceSubType ==
+                                  self._REMOTEFX_DISP_CTRL_RES_SUB_TYPE]
+
+        if not remotefx_disp_ctrl_res:
+            # VM does not have RemoteFX configured.
+            return
+
+        # we need to remove the RemoteFX display controller first.
+        self._jobutils.remove_virt_resource(remotefx_disp_ctrl_res[0])
+
+        synth_disp_ctrl_res = self._get_new_resource_setting_data(
+            self._SYNTH_DISP_CTRL_RES_SUB_TYPE,
+            self._SYNTH_DISP_ALLOCATION_SETTING_DATA_CLASS)
+        self._jobutils.add_virt_resource(synth_disp_ctrl_res, vm)
+
+        if self._vm_has_s3_controller(vm_name):
+            s3_disp_ctrl_res = [r for r in rasds if r.ResourceSubType ==
+                                self._S3_DISP_CTRL_RES_SUB_TYPE][0]
+            s3_disp_ctrl_res.Address = self._DISP_CTRL_ADDRESS
             self._jobutils.modify_virt_resource(s3_disp_ctrl_res)
 
     def _vm_has_s3_controller(self, vm_name):
@@ -1122,11 +1222,6 @@ class VMUtils(baseutils.BaseUtilsVirt):
         disk_resource.HostResource = [new_disk_path]
         self._jobutils.modify_virt_resource(disk_resource)
 
-    def _get_virtual_system_type(self, is_planned_vm):
-        return (
-            self._VIRTUAL_SYSTEM_TYPE_PLANNED if is_planned_vm
-            else self._VIRTUAL_SYSTEM_TYPE_REALIZED)
-
     def add_pci_device(self, vm_name, vendor_id, product_id):
         """Adds the given PCI device to the given VM.
 
@@ -1144,3 +1239,14 @@ class VMUtils(baseutils.BaseUtilsVirt):
         """
         raise NotImplementedError(_('PCI passthrough is supported on '
                                     'Windows / Hyper-V Server 2016 or newer.'))
+
+    def remove_all_pci_devices(self, vm_name):
+        """Removes all the PCI devices from the given VM.
+
+        There are no PCI devices attached to Windows / Hyper-V Server 2012 R2
+        or older VMs.
+        """
+
+    def _set_vm_snapshot_type(self, vmsettings, snapshot_type):
+        # Supported on Windows Server 2016 or newer.
+        pass
