@@ -23,14 +23,27 @@ import re
 
 from eventlet import patcher
 from eventlet import tpool
+from oslo_utils import units
+import six
 
-from os_win._i18n import _
+from os_win._i18n import _, _LE
 from os_win import constants
 from os_win import exceptions
 from os_win.utils import _wqlutils
 from os_win.utils import baseutils
 from os_win.utils import jobutils
 
+_PORT_PROFILE_ATTR_MAP = {
+    "profile_id": "ProfileId",
+    "profile_data": "ProfileData",
+    "profile_name": "ProfileName",
+    "net_cfg_instance_id": "NetCfgInstanceId",
+    "cdn_label_id": "CdnLabelId",
+    "cdn_label_string": "CdnLabelString",
+    "vendor_id": "VendorId",
+    "vendor_name": "VendorName",
+}
+
 
 class NetworkUtils(baseutils.BaseUtilsVirt):
 
@@ -42,8 +55,10 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
     _ETHERNET_SWITCH_PORT = 'Msvm_EthernetSwitchPort'
     _PORT_ALLOC_SET_DATA = 'Msvm_EthernetPortAllocationSettingData'
     _PORT_VLAN_SET_DATA = 'Msvm_EthernetSwitchPortVlanSettingData'
+    _PORT_PROFILE_SET_DATA = 'Msvm_EthernetSwitchPortProfileSettingData'
     _PORT_SECURITY_SET_DATA = 'Msvm_EthernetSwitchPortSecuritySettingData'
     _PORT_ALLOC_ACL_SET_DATA = 'Msvm_EthernetSwitchPortAclSettingData'
+    _PORT_BANDWIDTH_SET_DATA = 'Msvm_EthernetSwitchPortBandwidthSettingData'
     _PORT_EXT_ACL_SET_DATA = _PORT_ALLOC_ACL_SET_DATA
     _LAN_ENDPOINT = 'Msvm_LANEndpoint'
     _STATE_DISABLED = 3
@@ -82,8 +97,10 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
     _switches = {}
     _switch_ports = {}
     _vlan_sds = {}
+    _profile_sds = {}
     _vsid_sds = {}
     _sg_acl_sds = {}
+    _bandwidth_sds = {}
 
     def __init__(self):
         super(NetworkUtils, self).__init__()
@@ -103,6 +120,13 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             "Microsoft:[0-9A-F-]*\\\\[0-9A-F-]*\\\\[0-9A-F-]",
             flags=re.IGNORECASE)
 
+        # map between switch port's InstanceID and their Port Profile settings
+        # data WMI objects.
+        for profile in self._conn.Msvm_EthernetSwitchPortProfileSettingData():
+            match = switch_port_id_regex.match(profile.InstanceID)
+            if match:
+                self._profile_sds[match.group()] = profile
+
         # map between switch port's InstanceID and their VLAN setting data WMI
         # objects.
         for vlan_sd in self._conn.Msvm_EthernetSwitchPortVlanSettingData():
@@ -117,11 +141,19 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             if match:
                 self._vsid_sds[match.group()] = vsid_sd
 
+        # map between switch port's InstanceID and their bandwidth setting
+        # data WMI objects.
+        bandwidths = self._conn.Msvm_EthernetSwitchPortBandwidthSettingData()
+        for bandwidth_sd in bandwidths:
+            match = switch_port_id_regex.match(bandwidth_sd.InstanceID)
+            if match:
+                self._bandwidth_sds[match.group()] = bandwidth_sd
+
     def update_cache(self):
         # map between switch port ID and switch port WMI object.
-        self._switch_ports = {
-            port.ElementName: port for port in
-            self._conn.Msvm_EthernetPortAllocationSettingData()}
+        self._switch_ports.clear()
+        for port in self._conn.Msvm_EthernetPortAllocationSettingData():
+            self._switch_ports[port.ElementName] = port
 
     def clear_port_sg_acls_cache(self, switch_port_name):
         self._sg_acl_sds.pop(switch_port_name, None)
@@ -296,8 +328,50 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
                 pass
 
         self._switch_ports.pop(switch_port_name, None)
+        self._profile_sds.pop(sw_port.InstanceID, None)
         self._vlan_sds.pop(sw_port.InstanceID, None)
         self._vsid_sds.pop(sw_port.InstanceID, None)
+        self._bandwidth_sds.pop(sw_port.InstanceID, None)
+
+    def set_vswitch_port_profile_id(self, switch_port_name, profile_id,
+                                    profile_data, profile_name, vendor_name,
+                                    **kwargs):
+        """Sets up the port profile id.
+
+        :param switch_port_name: The ElementName of the vSwitch port.
+        :param profile_id: The profile id to be set for the given switch port.
+        :param profile_data: Additional data for the Port Profile.
+        :param profile_name: The name of the Port Profile.
+        :param net_cfg_instance_id: Unique device identifier of the
+            sub-interface.
+        :param cdn_label_id: The CDN Label Id.
+        :param cdn_label_string: The CDN label string.
+        :param vendor_id: The id of the Vendor defining the profile.
+        :param vendor_name: The name of the Vendor defining the profile.
+        """
+        port_alloc = self._get_switch_port_allocation(switch_port_name)[0]
+        port_profile = self._get_profile_setting_data_from_port_alloc(
+            port_alloc)
+
+        new_port_profile = self._prepare_profile_sd(
+            profile_id=profile_id, profile_data=profile_data,
+            profile_name=profile_name, vendor_name=vendor_name, **kwargs)
+
+        if port_profile:
+            # Removing the feature because it cannot be modified
+            # due to a wmi exception.
+            self._jobutils.remove_virt_feature(port_profile)
+
+            # remove from cache.
+            self._profile_sds.pop(port_alloc.InstanceID, None)
+
+        try:
+            self._jobutils.add_virt_feature(new_port_profile, port_alloc)
+        except Exception as ex:
+            raise exceptions.HyperVException(
+                _LE('Unable to set port profile settings %(port_profile)s '
+                    'for port %(port)s. Error: %(error)s') %
+                dict(port_profile=new_port_profile, port=port_alloc, error=ex))
 
     def set_vswitch_port_vlan_id(self, vlan_id=None, switch_port_name=None,
                                  **kwargs):
@@ -362,6 +436,21 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             raise exceptions.HyperVException(
                 _('Port VLAN not found: %s') % switch_port_name)
 
+    def _prepare_profile_sd(self, **kwargs):
+        profile_id_settings = self._create_default_setting_data(
+            self._PORT_PROFILE_SET_DATA)
+
+        for argument_name, attr_name in _PORT_PROFILE_ATTR_MAP.items():
+            attribute = kwargs.pop(argument_name, None)
+            if attribute is None:
+                continue
+            setattr(profile_id_settings, attr_name, attribute)
+
+        if kwargs:
+            raise TypeError("Unrecognized attributes %r" % kwargs)
+
+        return profile_id_settings
+
     def _prepare_vlan_sd_access_mode(self, vlan_settings, vlan_id):
         if vlan_settings:
             # the given vlan_id might be None.
@@ -443,6 +532,10 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             raise exceptions.HyperVException(
                 _('Port Security Settings not found: %s') % switch_port_name)
 
+    def _get_profile_setting_data_from_port_alloc(self, port_alloc):
+        return self._get_setting_data_from_port_alloc(
+            port_alloc, self._profile_sds, self._PORT_PROFILE_SET_DATA)
+
     def _get_vlan_setting_data_from_port_alloc(self, port_alloc):
         return self._get_setting_data_from_port_alloc(
             port_alloc, self._vlan_sds, self._PORT_VLAN_SET_DATA)
@@ -451,6 +544,10 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         return self._get_setting_data_from_port_alloc(
             port_alloc, self._vsid_sds, self._PORT_SECURITY_SET_DATA)
 
+    def _get_bandwidth_setting_data_from_port_alloc(self, port_alloc):
+        return self._get_setting_data_from_port_alloc(
+            port_alloc, self._bandwidth_sds, self._PORT_BANDWIDTH_SET_DATA)
+
     def _get_setting_data_from_port_alloc(self, port_alloc, cache, data_class):
         if port_alloc.InstanceID in cache:
             return cache[port_alloc.InstanceID]
@@ -675,6 +772,81 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         """
         return [0] * len(sg_rules)
 
+    def set_port_qos_rule(self, port_id, qos_rule):
+        """Sets the QoS rule for the given port.
+
+        :param port_id: the port's ID to which the QoS rule will be applied to.
+        :param qos_rule: a dictionary containing the following keys:
+            min_kbps, max_kbps, max_burst_kbps, max_burst_size_kb.
+        :raises exceptions.HyperVInvalidException: if
+            - min_kbps is smaller than 10MB.
+            - max_kbps is smaller than min_kbps.
+            - max_burst_kbps is smaller than max_kbps.
+        :raises exceptions.HyperVException: if the QoS rule cannot be set.
+        """
+
+        # Hyper-V stores bandwidth limits in bytes.
+        min_bps = qos_rule.get("min_kbps", 0) * units.Ki
+        max_bps = qos_rule.get("max_kbps", 0) * units.Ki
+        max_burst_bps = qos_rule.get("max_burst_kbps", 0) * units.Ki
+        max_burst_sz = qos_rule.get("max_burst_size_kb", 0) * units.Ki
+
+        if not (min_bps or max_bps or max_burst_bps or max_burst_sz):
+            # no limits need to be set
+            return
+
+        if min_bps and min_bps < 10 * units.Mi:
+            raise exceptions.InvalidParameterValue(
+                param_name="min_kbps", param_value=min_bps)
+        if max_bps and max_bps < min_bps:
+            raise exceptions.InvalidParameterValue(
+                param_name="max_kbps", param_value=max_bps)
+        if max_burst_bps and max_burst_bps < max_bps:
+            raise exceptions.InvalidParameterValue(
+                param_name="max_burst_kbps", param_value=max_burst_bps)
+
+        port_alloc = self._get_switch_port_allocation(port_id)[0]
+        bandwidth = self._get_bandwidth_setting_data_from_port_alloc(
+            port_alloc)
+        if bandwidth:
+            # Removing the feature because it cannot be modified
+            # due to a wmi exception.
+            self._jobutils.remove_virt_feature(bandwidth)
+
+            # remove from cache.
+            self._bandwidth_sds.pop(port_alloc.InstanceID, None)
+
+        bandwidth = self._get_default_setting_data(
+            self._PORT_BANDWIDTH_SET_DATA)
+        bandwidth.Reservation = min_bps
+        bandwidth.Limit = max_bps
+        bandwidth.BurstLimit = max_burst_bps
+        bandwidth.BurstSize = max_burst_sz
+
+        try:
+            self._jobutils.add_virt_feature(bandwidth, port_alloc)
+        except Exception as ex:
+            if '0x80070057' in six.text_type(ex):
+                raise exceptions.InvalidParameterValue(
+                    param_name="qos_rule", param_value=qos_rule)
+            raise exceptions.HyperVException(
+                _LE('Unable to set qos rule %(qos_rule)s for port %(port)s. '
+                    'Error: %(error)s') %
+                dict(qos_rule=qos_rule, port=port_alloc, error=ex))
+
+    def remove_port_qos_rule(self, port_id):
+        """Removes the QoS rule from the given port.
+
+        :param port_id: the port's ID from which the QoS rule will be removed.
+        """
+        port_alloc = self._get_switch_port_allocation(port_id)[0]
+        bandwidth = self._get_bandwidth_setting_data_from_port_alloc(
+            port_alloc)
+        if bandwidth:
+            self._jobutils.remove_virt_feature(bandwidth)
+            # remove from cache.
+            self._bandwidth_sds.pop(port_alloc.InstanceID, None)
+
 
 class NetworkUtilsR2(NetworkUtils):
     _PORT_EXT_ACL_SET_DATA = 'Msvm_EthernetSwitchPortExtendedAclSettingData'
