@@ -13,27 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_config import cfg
-from oslo_log import log as logging
 from oslo_utils import importutils
 
 from os_win._i18n import _, _LW  # noqa
 from os_win import exceptions
 from os_win.utils import hostutils
 from os_win.utils.io import namedpipe
-from os_win.utils.storage.initiator import iscsi_cli_utils
-
-hyper_opts = [
-    cfg.BoolOpt('force_volumeutils_v1',
-                default=False,
-                help='DEPRECATED: Force V1 volume utility class',
-                deprecated_for_removal=True),
-]
-
-CONF = cfg.CONF
-CONF.register_opts(hyper_opts, 'hyperv')
-
-LOG = logging.getLogger(__name__)
 
 utils = hostutils.HostUtils()
 
@@ -86,6 +71,11 @@ utils_map = {
             'min_version': 6.2,
             'max_version': None,
             'path': 'os_win.utils.metrics.metricsutils.MetricsUtils'}},
+    'migrationutils': {
+        'MigrationUtils': {
+            'min_version': 6.2,
+            'max_version': None,
+            'path': 'os_win.utils.compute.migrationutils.MigrationUtils'}},
     'networkutils': {
         'NetworkUtils': {
             'min_version': 6.2,
@@ -142,7 +132,7 @@ utils_map = {
 }
 
 
-def _get_class(class_type):
+def _get_class(class_type, *args, **kwargs):
     if class_type not in utils_map:
         raise exceptions.HyperVException(_('Class type %s does '
                                            'not exist') % class_type)
@@ -157,15 +147,17 @@ def _get_class(class_type):
         if (utils_class['min_version'] <= windows_version and
                 (utils_class['max_version'] is None or
                  windows_version < utils_class['max_version'])):
-            return importutils.import_object(utils_class['path'])
+            return importutils.import_object(utils_class['path'],
+                                             *args, **kwargs)
 
     raise exceptions.HyperVException(_('Could not find any %(class)s class for'
-        'this Windows version: %(win_version)s')
-        % {'class': class_type, 'win_version': windows_version})
+                                       'this Windows version: %(win_version)s')
+                                     % {'class': class_type,
+                                        'win_version': windows_version})
 
 
 def get_vmutils(host='.'):
-    return _get_class(class_type='vmutils')
+    return _get_class(class_type='vmutils', host=host)
 
 
 def get_vhdutils():
@@ -192,10 +184,7 @@ def get_pathutils():
     return _get_class(class_type='pathutils')
 
 
-def get_iscsi_initiator_utils(use_iscsi_cli=False):
-    use_iscsi_cli = use_iscsi_cli or CONF.hyperv.force_volumeutils_v1
-    if use_iscsi_cli:
-        return iscsi_cli_utils.ISCSIInitiatorCLIUtils()
+def get_iscsi_initiator_utils():
     return _get_class(class_type='iscsi_initiator_utils')
 
 
@@ -233,3 +222,7 @@ def get_clusterutils():
 
 def get_dnsutils():
     return _get_class(class_type='dnsutils')
+
+
+def get_migrationutils():
+    return _get_class(class_type='migrationutils')
